<?php
/*
 * Fusio is an open source API management platform which helps to create innovative API solutions.
 * For the current version and information visit <https://www.fusio-project.org/>
 *
 * Copyright (c) Christoph Kappestein <christoph.kappestein@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Fusio\Cli\Command;

use Fusio\Cli\Service\Client;
use Symfony\Component\Console\Command\Command;

/**
 * HttpCommandAbstract
 *
 * @author  Christoph Kappestein <christoph.kappestein@gmail.com>
 * @license http://www.apache.org/licenses/LICENSE-2.0
 * @link    https://www.fusio-project.org/
 */
abstract class ClientCommandAbstract extends Command
{
    protected Client $client;

    public function __construct(Client $client)
    {
        parent::__construct();

        $this->client = $client;
    }

    protected function toInt($value): ?int
    {
        if (is_array($value)) {
            throw new \InvalidArgumentException('Provided an invalid value');
        }

        if ($value === null || $value === '') {
            return null;
        }

        return (int) $value;
    }

    protected function toString($value): ?string
    {
        if (is_array($value)) {
            throw new \InvalidArgumentException('Provided an invalid value');
        }

        if ($value === null || $value === '') {
            return null;
        }

        return (string) $value;
    }
}
