<?php
/*
 * Fusio is an open source API management platform which helps to create innovative API solutions.
 * For the current version and information visit <https://www.fusio-project.org/>
 *
 * Copyright (c) Christoph Kappestein <christoph.kappestein@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Fusio\Impl\Backend\Action\Database\Table;

use Fusio\Engine\ContextInterface;
use Fusio\Engine\ParametersInterface;
use Fusio\Engine\RequestInterface;
use Fusio\Impl\Backend\Action\Database\TableAbstract;

/**
 * GetAll
 *
 * @author  Christoph Kappestein <christoph.kappestein@gmail.com>
 * @license http://www.apache.org/licenses/LICENSE-2.0
 * @link    https://www.fusio-project.org
 */
class GetAll extends TableAbstract
{
    public function handle(RequestInterface $request, ParametersInterface $configuration, ContextInterface $context): mixed
    {
        $connection = $this->getConnection($request);
        $startIndex = (int) $request->get('startIndex');
        $count = (int) $request->get('count');
        $limit = 1024;

        $startIndex = $startIndex < 0 ? 0 : $startIndex;
        $count = $count >= 1 && $count <= $limit ? $count : 16;

        $tableNames = $connection->createSchemaManager()->listTableNames();

        $totalResults = count($tableNames);
        sort($tableNames);
        $tableNames = array_slice($tableNames, $startIndex, $count);

        $result = [];
        foreach ($tableNames as $tableName) {
            $result[] = [
                'name' => $tableName,
            ];
        }

        return [
            'totalResults' => $totalResults,
            'itemsPerPage' => $count,
            'startIndex' => $startIndex,
            'entry' => $result,
        ];
    }
}
